#!/usr/bin/env python3

""" Transfer BrainSaw data to a server mounted on the local machine

This function is a wrapper round rsync and is used for copying BakingTray
data to a server over a network connection.

Input arguments
- one or more paths to local directories which are to be copied
- the last path is the destination.

transferToServer ./localSampleA ./localSampleB /path/to/server

For more see transferToServer -h


Notes
If you have signed in via SSH and aren't in a tmux session, the function
asks for confirmation before continuing. If your ssh session breaks off
for some reason, then compression will fail. tmux is therefore recommended
in this situation.

"""


import os
import argparse
import re
import sys
from textwrap import dedent  # To remove common leading white-space
from btpytools import tools


def cli_parser():
    """
    Build and return an argument parser
    """

    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        prog="transferToServer",
        description=dedent(
            """\
                            Transfer BrainSaw data to a server mounted on the local machine

                            This function is a wrapper round rsync and is used for copying BakingTray
                            data to a server. It automatically does not copy uncompressed raw data or
                            the un-cropped stacks directory.

                            Input arguments
                            - one or more paths to local directories which are to be copied
                            - the last path is the destination.


                            Usage examples:

                            1. Transfer one sample plus any compressed raw data:
                               $ transferToServer ./XY_123 /mnt/datastor/user/path/

                            2. Transfer a directory containing two samples (after cropping) plus any compressed raw data.
                               You will be prompted whether you want the source data to be copied in the enclosing directory.
                               $ transferToServer ./USER_sampleA_sampleB/ /mnt/datastor/user/path/

                               Note, this is the same as doing:
                               $ transferToServer ./USER_sampleA_sampleB/sample1 ./USER_sampleA_sampleB/sample2 ./USER_sampleA_sampleB/rawData.tar.bz /mnt/datastor/user/path/

                            3. Simulate the transfer of one sample
                               $ transferToServer ./XY_123 /mnt/datastor/user/path/ -s

                            4. Run with different rsync flags
                               $ transferToServer ./XY_123 /mnt/datastor/user/path/ -r rv

                           """
        ),
        epilog=dedent(
            """\
                        Notes
                        If you have signed in via SSH and aren't in a tmux session, the function
                        asks for confirmation before continuing. If your ssh session breaks off
                        for some reason, then compression will fail. tmux is therefore recommended
                        in this situation.
                      """
        ),
    )

    parser.add_argument(
        dest="paths",
        type=str,
        nargs="+",
        help="One or more local paths followed by destination path",
    )

    parser.add_argument(
        "-s",
        "--simulate",
        dest="simulate",
        required=False,
        action="store_true",
        help="If supplied an rsync dry run is conducted and nothing is copied.",
    )

    parser.add_argument(
        "-r",
        "--rsync_flags",
        dest="rsync_flags",
        required=False,
        default="av",
        type=str,
        help='If supplied, this rsync flag is used. Default is "av". Other reasonable options include "rv".',
    )

    return parser


def check_directories(source_dirs, destination_dir):
    """
    Returns True if the source and destination directories are all valid.
    False otherwise. This allows main() to bail out if, for instance, any
    of the supplied paths do not exist.
    """
    fail = False
    for tPath in source_dirs:
        if not os.path.exists(tPath):
            print("%s does not exist" % tPath)
            fail = True

    # At least check if the target location is a directory
    if not os.path.isdir(destination_dir):
        print("%s is not a valid destination directory" % destination_dir)
        fail = True

    return fail


def user_specified_cropped_directories_individually(source_dirs, verbose=False):
    """
    Checks whether the user has manually supplied any multiple split sample directories
    from a cropped acquisition. e.g. Say we have a folder called 'acq_123', which
    contains cropped samples 'acq_1', 'acq_2', and 'acq_3'. The user could either provide
    'acq_123' as the source or they could provide a list of all three directories:
    'acq_123/acq1', etc. If they have done the latter, this function returns True.

    This function is used in conjunction with others to determine whether there is a
    chance that the user is accidentally failing to back up compressed raw data.

    Inputs:
    source_dirs - a list of source directories. If just one entry it can be a string.

    Outputs
    False means the user did not specify cropped directories individually.
    True means that at least two directories in the input list are from the same
    cropped acquisition.

    Notes:
    - Function returns true even if some samples nested within the parent directory
      were not supplied.
    """

    # If the list is only one directory long (or is a string) then we still need to check if it
    # is a cropped directory before deciding for sure wether to return True or False.
    if (
        isinstance(source_dirs, str)
        or isinstance(source_dirs, list)
        and len(source_dirs) == 1
    ):
        if tools.has_compressed_raw_data(os.path.split(source_dirs)[0]):
            # Parent directory contains a compressed archive. This makes it
            # likely this was a cropped directory and that the user could
            # potentially be failing to include the archive in the list of
            # to transfer (although we will check this elsewhere). If there is no
            # compressed archive this will return False, which is OK.
            return True
        else:
            return False

    # Remove everything from the list that is not a directory
    source_dirs = [x for x in source_dirs if os.path.isdir(x)]

    # Remove any directories that are definitely single samples or uncropped acquisitions
    source_dirs = remove_single_samples_from_list(source_dirs)
    if len(source_dirs) == 0:
        # If list is empty we have no individually specified directories
        return False

    # Now we handle the potential scenario where the user's current folder is a
    # data acquisition directory.
    if tools.is_data_folder(os.getcwd()):
        # If the current directory is a BakingTray data folder and there are multiple source
        # directories (if here theen there are) then the user has, indeed, specified directories
        # individually.
        if verbose:
            print("Individually specified because current dir is a data dir")
        return True

    # Generate a new version of source_dirs with the last element of each path removed
    trimmed_list = [os.path.dirname(x) for x in source_dirs]

    # Since the current directory is not a sample directory, list entries that are empty
    # are in the current directory and therefore are either single samples or acquisitions
    # with multiple samples. We therefore can remove these.
    trimmed_list = [x for x in trimmed_list if len(x) > 0]

    # If the list is now empty, then it contains no individually specified directories
    # from within a cropped acquisition. This is because a list like this will all end
    # up being empty after the above: ['./sample_01', 'sample_02']
    if len(trimmed_list) == 0:
        return False

    # If the list contains two or more non-unique paths, then original source list contained
    # individually specified directories from within a cropped acquisition.
    if len(set(trimmed_list)) != len(trimmed_list):
        # There are duplicates so user must have asked for this
        if verbose:
            print("Duplicates found: must be individually specified")
        return True

    # At this point we know that source_dirs was a directory list that probably something of
    # this sort: './acq_xy/sample1' , './acq_ab/sample1'. This is an unusual scenario and,
    # like in the if statement above, means the user is specifying directories from a cropped
    # acquisition individually. We therefore return True.
    return True


def dir_list_contains_compressed_archive(source_dirs):
    """
    Does the directory list in source_dirs contain at least one compressed raw data archive? The
    lack of a compressed archive in this list does not meant the compressed raw data are not going
    to be sent. It just reflects whether or not the user has explicitly asked to send the data.

    This function is used in conjunction with others to determine whether there is a chance that
    the user is accidentally failing to back up compressed raw data.

    Inputs:
    source_dirs - a list of source directories

    Outputs
    True means the directory list in source_dirs contains a raw data archive. False otherwise.
    """

    compressed_archives = [
        x
        for x in source_dirs
        if isinstance(re.match(r".*rawData.*\.tar\.[bg]z", x), re.Match)
    ]

    if len(compressed_archives) > 0:
        return True
    else:
        return False


def issue_warning_if_compressed_data_will_not_be_sent(source_dirs, verbose=False):
    """
    If the user is running transferToServer in a way that would cause raw data not be transfered,
    we return True. If the source directory list appears safe we return False.
    """

    if not user_specified_cropped_directories_individually(source_dirs):
        # Then we are definitely fine and we can return false
        if verbose:
            print("Directories not specified individually")

        return False

    # If we are here, there are indivually specified directories
    if verbose:
        print("Individually specified directories found")

    # If the user specified the directories individually we are still "safe" so long as either
    # a) They included the compressed archive in the transfer list (source_dirs).
    # b) There is no compressed archive (meaning the user does not want to back up raw data)
    if dir_list_contains_compressed_archive(source_dirs):
        if verbose:
            print("List contains a compressed archive")
        return False

    # Go through each directory in the list. If it is The following
    dir_to_test = os.path.split(source_dirs[0])[0]
    if not tools.has_compressed_raw_data(dir_to_test):
        if verbose:
            print("No raw data archive found in directory %s" % dir_to_test)
        return False

    # If we are here, there could be compressed raw data that are not being sent
    return True


def remove_single_samples_from_list(source_dirs, verbose=False):
    """
    Remove from list source_list all folders that are definitely not an individually specified
    directory. Return this list.
    """

    if not isinstance(source_dirs, list):
        return

    # Remove everything from the list that is not a directory
    source_dirs = [x for x in source_dirs if os.path.isdir(x)]

    if verbose:
        print("\nInitial dir list:\n")
        print(source_dirs)

    # If it has a raw data archive or does not have a cropped directory with the raw data
    # then it is likely a non separately specified directory. So we can remove it from the list.
    # keep removing. If list empty user had none specified indiviudally.
    for t_dir in source_dirs:
        if (
            tools.has_compressed_raw_data(t_dir)
            or tools.has_raw_data(t_dir)
            or tools.has_uncropped_stitched_images(t_dir)
        ):
            if verbose:
                print("Removed %s" % t_dir)
            source_dirs = [x for x in source_dirs if x != t_dir]  # remove t_dir

        if verbose:
            print("Tested %s. length=%s" % (t_dir, len(source_dirs)))

    if len(source_dirs) == 0:
        return list()

    return source_dirs


def main():
    """
    main()
    """

    # Process arguments and set up
    args = cli_parser().parse_args()

    if len(args.paths) < 2:
        print(
            "\n\n ERROR: transferToServer requires at least one local path to copy and a destination\n"
        )
        print(' See "transferToServer -h"\n')
        sys.exit()

    main_rsync_switch = args.rsync_flags  # This is the flag used by rsync
    if not main_rsync_switch.startswith("-"):
        main_rsync_switch = "-%s" % main_rsync_switch

    # Append "n" for dry-run rsync mode if the user asked for this with the -s flag at CLI
    if args.simulate:
        main_rsync_switch += "n"

    source_dirs = args.paths[0:-1]  # One or more files or folders to copy
    destination_dir = args.paths[-1]  # Where we will copy to

    # Bail out if any of the supplied paths do not exist
    if check_directories(source_dirs, destination_dir):
        sys.exit()

    # Remove trailing slash from data directories that don't contain data sub-directories
    for ii, t_dir in enumerate(source_dirs):
        if tools.is_data_folder(t_dir) and not tools.contains_data_folders(t_dir):
            # If here, tDIR is a sample folder without sub-folders. If there is a
            # trailing slash then we should delete it. Always.
            if t_dir[-1] == os.path.sep:
                source_dirs[ii] = t_dir[0:-1]

        elif tools.contains_data_folders(t_dir):
            # If here, tDIR contains sub-folders which are sample folders. Thus is likely
            # contains cropped data. We again delete the trailing slash but we give the
            # user the option to add it back, copying the *contents* to the destination
            # rather than in the enclosing folder.
            if t_dir[-1] == os.path.sep:
                source_dirs[ii] = t_dir[0:-1]

            print('\nDirectory "%s" contains multiple samples.' % t_dir)
            print(
                "Do you want to keep samples in their enclosing directory (%s) when on the server?"
                % t_dir
            )
            if not tools.query_yes_no(""):
                # Add trailing slash, if user does not want to keep enclosing folder.
                source_dirs[ii] = source_dirs[ii] + os.path.sep

    print("")

    # Check whether any of the directories we plan to copy already exist at the destination
    safe_to_copy = True
    for ii, t_dir in enumerate(source_dirs):

        if t_dir[-1] != os.path.sep:
            # Look for this directory at the destination
            destination_path = os.path.join(destination_dir, t_dir)
            if os.path.exists(destination_path):
                safe_to_copy = False
                print(
                    '===>>> WARNING!! "%s" already exists in "%s"!! Proceeding will over-write its contents <===='
                    % (t_dir, destination_dir)
                )
        else:
            # Look for the *contents* of this directory at the destination
            dir_contents = os.listdir(t_dir)
            for jj, sub_dir in enumerate(dir_contents):
                if sub_dir == "rawData" or "_DELETE_ME" in sub_dir:
                    continue
                destination_path = os.path.join(destination_dir, sub_dir)
                if os.path.exists(destination_path):
                    safe_to_copy = False
                    print(
                        '===>>> WARNING!! "%s" already exists in "%s"!! Proceeding will over-write its contents <===='
                        % (sub_dir, destination_dir)
                    )

    if not safe_to_copy:
        print("\n IS IT OK TO PROCEED DESPITE THE ABOVE WARNINGS?")
        if not tools.query_yes_no(""):
            sys.exit()

    # Ask for confirmation before starting
    print("\nPerform the following transfer?")
    for ii, t_dir in enumerate(source_dirs):
        if t_dir[-1] != os.path.sep:
            print('Copy directory "%s" to location "%s"' % (t_dir, destination_dir))
        else:
            dir_contents = os.listdir(t_dir)
            for jj, sub_contents in enumerate(dir_contents):
                if sub_contents == "rawData" or sub_contents.endswith("_DELETE_ME"):
                    pass
                else:
                    print('Copy "%s" to "%s"' % (sub_contents, destination_dir))

    # Build the rsync command string
    cmd = "rsync %s --progress --exclude rawData --exclude *_DELETE_ME_* %s %s " % (
        main_rsync_switch,
        " ".join(source_dirs),
        destination_dir,
    )

    print("Using command %s" % cmd)
    if not tools.query_yes_no(""):
        sys.exit()

    # Start the transfer
    os.system(cmd)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

""" Compress raw data from a BakingTray acquisition to a tar.bz

Usage
cd to sample directory and run this command:
$ cd /mnt/data/somesample
$ compressRawData


Notes
If you have signed in via SSH and aren't in a tmux session, the function
asks for confirmation before continuing. If your ssh session breaks off
for some reason, then compression will fail. tmux is therefore recommended
in this situation.

The compression does not run if there is insufficient disk space. It does not run if
free_disk_space_GB < (uncompressed raw_data_size_GB * 0.7) + 500


Runs much faster with parallel bzip (lbzip2) installed. If this is missing, the tool reverts
to regular gzip.

"""


import os
import argparse
import time
import sys
from datetime import datetime
from glob import glob
from textwrap import dedent  # To remove common leading white space
from btpytools import tools, recipe


def cli_parser():
    """
    Build and return an argument parser
    """

    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        prog="compressRawData",
        description=dedent(
            """\
                            Compress a raw BrainSaw data directory

                            Usage:

                              cd to sample directory and run this command:
                              $ cd /mnt/data/somesample
                              $ compressRawData
                             """
        ),
        epilog=dedent(
            """\
                        Notes
                        If you have signed in via SSH and aren't in a tmux session, the function
                        asks for confirmation before continuing. If your ssh session breaks off
                        for some reason, then compression will fail. tmux is therefore recommended
                        in this situation.

                        Runs much faster with parallel bzip (lbzip2) installed. If this is missing,
                        the tool reverts to regular gzip."""
        ),
    )

    parser.add_argument(
        "-s",
        "--simulate",
        dest="simulate",
        required=False,
        action="store_true",
        help="If supplied only progress text messages are supplied. No compression run.",
    )

    return parser


def main():
    """
    Main function: runs the compression
    """

    args = cli_parser().parse_args()

    if args.simulate:
        print("\n\nRUNNING IN SIMULATE MODE\n")

    if not tools.has_raw_data():
        print("No rawData folder found in", os.getcwd())
        sys.exit()

    # Is there sufficient disk space to run the compression?
    print("Calculating raw data size... Please wait.")
    free_GB = tools.get_free_disk_space_in_GB()
    raw_data_GB = tools.get_dir_size_in_GB(tools.RAW_DATA_DIR, fast_raw_data=True)

    # There should be at least this many GB free after compression
    free_size_buffer = 500

    # If there is less space available than the size of the raw data we quit
    if free_GB < (raw_data_GB * 0.7):
        print(
            "Compression adds about %d GB but there is only %d free. Free space and try again."
            % (int(raw_data_GB), int(free_GB))
        )
        sys.exit()

    # If there is less space available than the size of the raw data plus the buffer we also quit
    # but display a slightly different message.
    if free_GB < (raw_data_GB * 0.7) + free_size_buffer:
        print(
            "Compression adds about %d GB and there is only %d free."
            % (int(raw_data_GB), int(free_GB))
        )
        print(
            "For safety compression will not proceed. Please free space and try again."
        )
        sys.exit()

    print(
        "OK! Compressed archive will be about %d GB and so %d GB will be free after compression."
        % (int(raw_data_GB * 0.7), int(free_GB - raw_data_GB))
    )

    # The sample ID is used to name the raw data directory. We get this information from
    # the recipe file. We look for this in the current directory and, failing that, in the
    # cropped data directory. If neither exists, we use the current directory name instead.

    # If there is no recipe file in the current directory, look for one in the cropped data dir.
    # if also that is missing, we use as the

    copy_files_from_uncropped = False  # Do we need to temporarily copy settings files out of the uncropped data dir?

    if tools.has_recipe_file():
        sample_name = recipe.sample_id()
    elif tools.has_uncropped_stitched_images():
        # Can we get sample ID from a recipe file in the uncropped stacks directory?
        uncropped_dir = glob(tools.UNCROPPED_WILDCARD)[0]
        if tools.has_recipe_file(uncropped_dir):
            # We get the sample ID from the uncropped data directory, so we also want to later
            # copy other settings files temporarily into the current directory so they
            # get archived
            print(
                "Getting sample information from meta-data files in uncropped data directory"
            )
            copy_files_from_uncropped = True
            sample_name = recipe.sample_id(uncropped_dir)
        else:
            # We find no recipe file anywhere but there were were raw data. Let's, therefore,
            # use the current working directory and the date to build the sample name
            print("No meta data files. Making up a sample ID from directory name.")
            cwd = os.getcwd()
            sample_name = cwd.split(os.path.sep)[-1]

            now = datetime.now()
            sample_name = sample_name + now.strftime("_%Y%m%d")

    print("Found rawData directory for sample " + sample_name)

    # Begin compression, but warn if we aren't in a tmux session if this is an SSH connection
    if tools.in_ssh_session() and not tools.in_tmux_session():
        if not tools.query_yes_no(
            "\nYou are logged in via SSH but are not in a tmux session, proceed anyway?",
            "no",
        ):
            print("Not proceeding with compression.\n")
            sys.exit()

    # We will also copy metadata
    meta_data_file_names = "scanSettings.mat *.yml *.txt *.ini"  # wildcards for files other than raw data that we will compress

    # Check whether we have lbzip2 and use it if so
    out = os.system("lbzip2 -h > /dev/null")
    has_lbzip2 = out == 0

    if has_lbzip2:
        tar_switches = "-I lbzip2 -cvf"
        compressed_rawdata_name = sample_name + "_rawData.tar.bz"

    else:
        tar_switches = "-zcvf"
        compressed_rawdata_name = sample_name + "_rawData.tar.gz"
        print(
            "\n\n** WARNING -- parallel compression tool lbzip2 not found. Using much slower single-threaded gzip.\n\n"
        )
        time.sleep(3)

    # Start compressing with parallel bzip
    compress_cmd = "tar %s %s %s ./rawData " % (
        tar_switches,
        compressed_rawdata_name,
        meta_data_file_names,
    )

    print("\nRunning compression command: " + compress_cmd + "\n")
    time.sleep(1)

    if not args.simulate:
        # If necessary we copy the meta-data files temporarily out of the uncropped stacks folder
        if copy_files_from_uncropped:
            print(
                "Temporarily moving meta-data from uncropped directory into main directory"
            )
            for these_files in meta_data_file_names.split(" "):
                os.system("cp %s/%s ./" % (uncropped_dir, these_files))

        # Run the compression command
        os.system(compress_cmd)

        # If needed, remove the files that we added. The following is slightly dangerous, but if normal
        # usage there should be no problem with the wild card delete.

        if copy_files_from_uncropped:
            print("\nTidying up temporarily copied files")
            os.system("rm -f %s" % meta_data_file_names)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

""" Summarize to screen all acquisitions in the current directory

Usage
cd to sample directory and run this command:
$ cd /mnt/data/
$ summariseAcquisitions

This function is automatically added as a console-accessible command during pip install
"""


import os

from btpytools import tools, recipe


def main():
    dirs = next(os.walk("."))[1]

    for tDir in dirs:

        if tools.is_data_folder(tDir):
            print(
                "%s is a BakingTray Acq started at %s"
                % (tDir, recipe.acq_start_time(tDir))
            )
        elif tools.contains_data_folders(tDir):
            print("%s contains BakingTray Acq sub-dirs" % tDir)
        else:
            print("%s is not a BakingTray acq dir" % tDir)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

""" Run brainreg on a BakingTray acquisition

Assumes the brain was acquired such that the images stacks are orderd posterior to anterior
and the dorsal surface is at the top of the images


Usage
cd to sample directory and run this command:
$ cd /mnt/data/
$ brainreg_bt

This function is automatically added as a console-accessible command during pip install
"""

import os
import argparse
import re
from btpytools import tools, recipe
from brainreg.paths import Paths
from brainreg.main import main as register
from brainreg import cli
from imlib.general.numerical import check_positive_int


# Defaults for registration

DEFAULT_VOXEL_SIZE = 25
DEFAULT_CHANNEL = "red"
DEFAULT_ORIENTATION = "psl"
DEFAULT_ATLAS = "allen_mouse"


def main():

    # Parse input arguments
    args = get_arg_parser()
    args = args.parse_args()
    arg_groups = cli.get_arg_groups(args, cli.register_cli_parser())

    # Confirm that we are in a BakingTray acquisition directory that contains downsampled
    # stacks. Bail out if this is not the case.
    if not confirm_directory():
        return

    # Get the voxel size, providing a CLI UI if needed.
    (vox_size, ds_stacks_chosen_vox_size) = choose_vox_size(DEFAULT_VOXEL_SIZE, args)
    if ds_stacks_chosen_vox_size is False:
        return

    args.voxel_sizes = tuple([vox_size] * 3)  # set argument based on user choice

    # Get the color size, providing a CLI UI if needed.
    chosen_ds = choose_color_chan(DEFAULT_CHANNEL, args, ds_stacks_chosen_vox_size)
    if chosen_ds is False:
        return

    args.bt_channel = chosen_ds["channelname"]  # set argument based on user choice

    # Prepare variables for registration
    atlas_name = "%s_%dum" % (DEFAULT_ATLAS, args.bt_voxel)
    vol_to_register = chosen_ds["path2stack"]

    print(
        "\n\nRegistering data for sample %s with %d micron downsampled %s stack."
        % (chosen_ds["samplename"], vox_size, args.bt_channel)
    )

    input("Press return to start\n")

    # Make the directory/set the directory
    reg_dir = tools.make_reg_dir()
    args.brainreg_directory = Paths(reg_dir)

    # Run the registration using brainreg
    args, additional_images_downsample = cli.prep_registration(args)

    register(
        atlas_name,
        args.orientation,
        vol_to_register,
        args.brainreg_directory,
        args.voxel_sizes,
        arg_groups["NiftyReg registration backend options"],
        sort_input_file=args.sort_input_file,
        n_free_cpus=args.n_free_cpus,
        additional_images_downsample=additional_images_downsample,
        backend=args.backend,
        debug=args.debug,
        save_original_orientation=args.save_original_orientation,
    )


def get_arg_parser():
    args = cli.register_cli_parser()

    # Add new options for BakingTray
    cli_args = args.add_argument_group("BakingTray data registration options")

    cli_args.add_argument(
        "-V",
        dest="bt_voxel",
        required=False,
        default=DEFAULT_VOXEL_SIZE,
        type=check_positive_int,
        help="Downsampled stack voxel size to register.",
    )

    cli_args.add_argument(
        "-C",
        "--channel",
        dest="bt_channel",
        required=False,
        default=DEFAULT_CHANNEL,
        type=str,
        help="Downsampled stack channel to register.",
    )

    cli_args.add_argument(
        "-D",
        "--use_defaults",
        dest="bt_use_defaults",
        required=False,
        action="store_true",
        help="If supplied, the CLI user choose is skipped and registration runs with defaults",
    )
    # Populate default values just in case this matters
    args.set_defaults(
        voxel_sizes=tuple([DEFAULT_VOXEL_SIZE] * 3),
        brainreg_directory=Paths(tools.ROOT_REG_DIR),
    )

    # Make these (and a few others) optional and also hide them from the help text
    # All the following will potentially need updating after the user has changed settings (TODO)
    for action in args._actions:
        if action.dest in [
            "voxel_sizes",
            "brainreg_directory",
            "image_paths",
            "downsample_images",
        ]:
            action.required = False
            action.help = argparse.SUPPRESS

    # Make orientation optional only (not invisiable) and set default
    for action in args._actions:
        if action.dest in [
            "orientation",
        ]:
            action.required = False

    args.set_defaults(orientation=DEFAULT_ORIENTATION)

    return args


def confirm_directory():
    """
    Confirm that we are in a BakingTray acquisition directory that contains downsampled
    stacks. Bail out if this is not the case.
    """

    if not tools.is_data_folder():
        print("\nCurrent folder is not a BakingTray acquisition directory\n")
        return False

    if not tools.has_downsampled_stacks():
        print("\nCurrent acquisition directory has no downsampled stacks\n")
        return False

    return True


def choose_vox_size(default, args):
    """
    Confirm that the desired voxel size exists. If the user
    has not changed the default value then we go through the a CLI
    interface to allow them to manually choose it or confirm it.
    """
    # Confirm that desired channel and voxel sizes exist

    ds_stacks = tools.available_downsampled_volumes()
    unique_voxel_sizes = list(set([x["voxelsize"] for x in ds_stacks]))

    # If the default voxel size matches that in the arg structure then
    # the user has not changed it. In this case we present the CLI UI
    if default == args.bt_voxel:
        run_cli_ui = True
    else:
        run_cli_ui = False

    # But if user supplies -D for using default values then this takes precedence and no UI
    if args.bt_use_defaults:
        run_cli_ui = False

    # Run the CLI UI
    msg = ""
    if run_cli_ui:

        msg = "Select downsampled stack voxel size for registration.\n(Enter a number and press return)\n"
        n = 1
        for t_vox in unique_voxel_sizes:
            msg += "%d. %d micron\n" % (n, t_vox)
            if default == t_vox:
                default_index = n - 1
            n += 1

        valid_answers = list(range(1, len(unique_voxel_sizes) + 1))
        valid_answers = [str(x) for x in valid_answers]
        error_str = "Enter a number between 1 and %d and press return" % (n - 1)
        reply = tools.cli_question(msg, valid_answers, default_index, error_str)

        chosen_voxel_size = unique_voxel_sizes[int(reply) - 1]
    else:
        chosen_voxel_size = args.bt_voxel

    ds_stacks_chosen_vox_size = [
        x for x in ds_stacks if x["voxelsize"] == chosen_voxel_size
    ]

    return (chosen_voxel_size, ds_stacks_chosen_vox_size)


def choose_color_chan(default, args, ds_stacks_chosen_vox_size):
    """
    Chose the desired channel from those available at the desired voxel size
    """

    # If the default channel matches that in the arg structure then
    # the user has not changed it. In this case we present the CLI UI
    if default == args.bt_channel:
        run_cli_ui = True
    else:
        run_cli_ui = False

    # But if user supplies -D for using default values then this takes precedence and no UI
    if args.bt_use_defaults:
        run_cli_ui = False

    # Run the CLI UI
    # These are the unique channel names
    chan_names = [x["channelname"] for x in ds_stacks_chosen_vox_size]

    msg = ""
    if run_cli_ui:
        msg = "Choose channel for registration.\n"
        n = 1
        for t_chan in chan_names:
            msg += "%d. %s channel\n" % (n, t_chan)
            if default == t_chan:
                default_index = n - 1
            n += 1

        valid_answers = list(range(1, len(chan_names) + 1))
        valid_answers = [str(x) for x in valid_answers]
        error_str = "Enter a number between 1 and %d and press return" % (n - 1)
        reply = tools.cli_question(msg, valid_answers, default_index, error_str)

        chosen_channel = chan_names[int(reply) - 1]
    else:
        chosen_channel = args.bt_channel

    # Extract dictionary element with this channel. There will be only one.
    chosen_ds = [
        x for x in ds_stacks_chosen_vox_size if x["channelname"] == chosen_channel
    ][0]

    return chosen_ds


if __name__ == "__main__":
    main()



import os
import re

"""
Simple tools for assisting with troublehooting btrfs RAID arrays.
Not bulletproof!

must install smartmontools and btrfs-progs
"""


def return_mount_point():
    """
    Assumes fstab has one btrfs mount point and returns the path to this.

    Inputs
    none

    Outputs
    A string corresponding to the btrfs mouint pount
    """
    with open("/etc/fstab", "r") as stream:
        contents = stream.read()
        match = re.search(r" +(/.*? ) +btrfs +", contents)
        return match.group(1)


def show(quiet=False, verbose=False):
    """
    Run btrfs fi show on btrfs array. By default print to screen the standard output from
    the command.
    """
    cmd = "sudo btrfs fi show %s" % return_mount_point()
    if verbose:
        os.system(cmd)
    std_out = os.popen(cmd).read()

    if not quiet:
        print(std_out)

    return std_out


def return_hdds_as_list():
    """
    Return btrfs devices on array as a list of strings
    """
    txt = show(quiet=True)
    match = re.findall("/dev/.*", txt)
    return match


def power_on_time(quiet=False):
    """
    Power on time of each device in the array
    """
    hdds = return_hdds_as_list()
    power_on_days = list()
    for t_hdd in hdds:
        cmd = "sudo smartctl --all %s | grep Power_On_Hours" % t_hdd
        std_out = os.popen(cmd).read()
        match = re.match(r".*- + (\d+)", std_out)

        power_on_days.append(int(match.group(1)) / 24)

        if not quiet:
            print("%s %d days" % (t_hdd, int(power_on_days[-1])))

    return (hdds, power_on_days)


def disks_older_than(threshold_age=700):
    """
    Print to screen a list of disks older than a certain number of days.
    By default 700 days is the threshold.

    Inputs
    threhold_age - scalar defining the number of days older than which we print
                  the disk name
    """

    disk_data = power_on_time(quiet=True)

    for t_disk in zip(disk_data[0], disk_data[1]):
        if t_disk[1] > threshold_age:
            print("%s %d days" % (t_disk[0], int(t_disk[1])))



"""
General purpose tools for handling BakingTray data directories
"""

import sys
import os
import re
from glob import glob
from pathlib import Path
import shutil
from datetime import datetime

# Define variables that will be common across functions
STITCHED_IMAGE_DIR = "stitchedImages_*"
RAW_DATA_DIR = "rawData"
DOWNSAMPLED_DIR = "downsampled_stacks"
DOWNSAMPLED_STACK_SUB_DIR = "*_micron"  # Sub-directories in DOWNSAMPLED_DIR
DOWNSAMPLED_STACK_LOG_FILE = "ds_*.txt"  # Wildcard for downsampled stacks
UNCROPPED_WILDCARD = "./Uncropped*_DELETE_ME_DELETE_ME"
RECIPE_WILDCARD = "recipe*.yml"

# Related to registration directories.
ROOT_REG_DIR = "registration"
REG_DIR_REGEX = r"reg_\d{2}__\d{4}_\d{2}_\d{2}_\w"


def has_raw_data(t_path=""):
    """Check if current directory (or that defined by t_path) contains a
    rawData directory. Returns True if present, False if absent.
    """
    t_path = os.path.join(t_path, RAW_DATA_DIR)
    return os.path.isdir(t_path)


def has_compressed_raw_data(t_path=""):
    """Check if current directory (or that defined by t_path) has
    compressed raw data. Returns True if present, False if absent
    """
    t_path = os.path.join(t_path, "*rawData*.tar.[gb]z")
    return file_glob_exist(t_path)


def has_recipe_file(t_path=""):
    """Check if current directory (or that defined by t_path) contains
    a recipe file. Returns True if present, False if absent.
    """
    t_path = os.path.join(t_path, RECIPE_WILDCARD)
    return file_glob_exist(t_path)


def has_scan_settings(t_path=""):
    """Check if current directory (or that defined by t_path) contains
    a scanSettings.mat file. Returns True if present, False if absent.
    """
    t_path = os.path.join(t_path, "scanSettings.mat")
    return os.path.isfile(t_path)


def has_stitched_images_directory(t_path=""):
    """Check if current directory (or that defined by t_path) contains a
    stitched image directory Returns True if present, False if absent.
    """
    t_path = os.path.join(t_path, STITCHED_IMAGE_DIR)
    return file_glob_exist(t_path)


def has_stitched_stacks(t_path=""):
    """Check if current directory (or that defined by t_path) contains
    stitched tiff stacks. Returns True if present, False if absent.
    """
    t_path = os.path.join(t_path, "./*_chan_0[1-9].tiff")
    return file_glob_exist(t_path)


def has_downsampled_stacks(t_path="", verbose=False):
    """Check if current directory (or that defined by t_path) contains a
    downsampled_stacks directory. Returns True if present, False if absent.
    """

    # Is there a downsampled stack dir?
    t_path = os.path.join(t_path, DOWNSAMPLED_DIR)

    if not file_glob_exist(t_path):
        if verbose:
            print("Can not find path %s" % t_path)

        return False

    # Does the downsampled stack directory contain downsampled stack directories?
    if not file_glob_exist(os.path.join(t_path, DOWNSAMPLED_STACK_SUB_DIR)):
        if verbose:
            print(
                "Downsampled stack directory does not contain downsampled stack sub-dirs"
            )
        return False

    return file_glob_exist(t_path)


def has_uncropped_stitched_images(t_path=""):
    """Check if current directory (or t_path) contains a stitched image
    directory. Returns True if present, False if absent.
    """
    t_path = os.path.join(t_path, UNCROPPED_WILDCARD)
    return file_glob_exist(t_path)


def is_data_folder(dirToTest=""):
    """is directory "dirToTest" a BakingTray data directory?
    i.e. it satisfies the following criteria:
     - contains a recipe YML
     - contains a scanSettings.mat file
     - contains one of: stitchedImages directory, stitched stacks,
                     rawData directory, compressed rawData
    """
    if (has_recipe_file(dirToTest) and has_scan_settings(dirToTest)) and (
        has_stitched_stacks(dirToTest)
        or has_stitched_images_directory(dirToTest)
        or has_raw_data(dirToTest)
        or has_compressed_raw_data(dirToTest)
    ):
        return True
    else:
        return False


def contains_data_folders(dirToTest):
    """returns true if dirToTest contains sub-directories that are
    BakingTray data folders.
    """
    subDirs = next(os.walk(dirToTest))[1]
    if len(subDirs) == 0:
        return False
    for tDir in subDirs:
        if is_data_folder(os.path.join(dirToTest, tDir)):
            return True

    # If we're here then there are no data-containing subdirs
    return False


def available_downsampled_volumes(in_path="", verbose=False):
    """Check if current directory (or that defined by t_path) contains a ownsampled stacks
    directory and returns a list of dictionaries listing the available downsampled data.
    If none are present returns False.
    """
    if not has_downsampled_stacks(in_path, verbose):
        return False

    # This defines the downsampled stack directory and what the downsampled
    # stacks are called. The following line can have two layers of wildcards.
    # e.g. downsampled_stacks/*_micron/ds_*.txt
    in_path = os.path.join(
        in_path, DOWNSAMPLED_DIR, DOWNSAMPLED_STACK_SUB_DIR, DOWNSAMPLED_STACK_LOG_FILE
    )
    paths_to_downsampled_stacks = glob(in_path)

    if len(paths_to_downsampled_stacks) < 1:
        if verbose("No downsampled stacks found"):
            return False

    out = []
    for t_path in paths_to_downsampled_stacks:
        t_log = read_downsample_log_file(t_path)
        out.append(t_log)

    return out


def read_downsample_log_file(pathToFile=""):
    """Parse StitchIt downsampled data file and return as a dictionary
    pathToFile is the path to a dowsampled log file returns False if
    no data are found in the defined path or multiple paths were supplied.
    """

    print("READING FILE %s" % pathToFile)
    if len(pathToFile) < 1:
        return False
    elif isinstance(pathToFile, list) and len(pathToFile) == 1:
        pathToFile = pathToFile[0]
    elif isinstance(pathToFile, list) and len(pathToFile) > 1:
        return False

    with open(pathToFile) as ff:
        lines = ff.readlines()

    # Make a dictionary from the file contents
    out = dict()

    # Sample name
    s = re.search("Downsampling (.*)", get_line_with_substr(lines, "Downsampling"))
    out["samplename"] = s.group(1)

    # Downsampled file name
    s = re.search(
        "downsample file name: (.*)",
        get_line_with_substr(lines, "downsample file name"),
    )
    out["path2stack"] = s.group(1)

    # Acquisition date
    s = re.search("Acquired on: (.*)", get_line_with_substr(lines, "Acquired"))
    out["acqdate"] = s.group(1)

    # Get the voxel size from the directory name
    s = re.search(r".*/(\d+)_micron/", pathToFile)
    out["voxelsize"] = int(s.group(1))

    # Get the channel index from the file name
    s = re.search(r".*_ch(0\d+)", pathToFile)
    out["channelindex"] = int(s.group(1))


    # Get the channel friendly name from the file name
    s = re.search(r".*_ch\d+_chan_\d_(.*)\.t", pathToFile)
    if s is not None:
        out["channelname"] = s.group(1)
    else:
        out["channelname"] = str(out["channelindex"])

    return out


"""
The following are very general purpose and not specific to BakingTray
"""


def file_glob_exist(t_path):
    if len(glob(t_path)) == 0:
        return False
    else:
        return True


def query_yes_no(question, default="yes"):
    """Ask a yes/no question via raw_input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    The "answer" return value is True for "yes" or False for "no".
    """
    valid = {"yes": True, "y": True, "ye": True, "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = input().lower()
        if default is not None and choice == "":
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' " "(or 'y' or 'n').\n")


def in_tmux_session():
    """Return true if we are running in a tmux session"""
    if os.popen("echo $TERM").read().strip() == "screen":
        return True
    else:
        return False


def in_ssh_session():
    """Return true if we are in an SSH session"""
    if len(os.popen("echo $SSH_CONNECTION").read().strip()) > 0:
        return True
    else:
        return False


def get_line_with_substr(allLines, subStr):
    """Return line from list of strings that contains a substring
    False if nothing is found
    Returns a list if multiple lines found
    """

    tLine = [myS for myS in allLines if subStr in myS]

    if len(tLine) < 1:
        return False

    if len(tLine) == 1:
        return tLine[0].rstrip()
    else:
        return tLine


def get_dir_size_in_GB(t_path, fast_raw_data=False):
    """
    Return the size of directory t_path in GB.
    Return False if directory dow not exist

    If fast_raw_data is true, we assume this is a raw data
    directory and measure the size of every 10th sub-dir then
    multiply by 10 to estimate the full size.
    """

    if not file_glob_exist(t_path):
        return False

    if fast_raw_data:
        t_path = Path(t_path).glob("*-*1/*")
    else:
        t_path = Path(t_path).glob("**/*")

    size_in_bytes = sum(f.stat().st_size for f in t_path if f.is_file())

    if fast_raw_data:
        size_in_bytes = size_in_bytes * 10  # Because we measured every 10th directory
        size_in_bytes = size_in_bytes * 1.03  # Because we skipped some directories

    return size_in_bytes / 1024**3


def get_free_disk_space_in_GB(t_path="."):
    """
    Return free disk space in GB. Uses volume associated with current path if no
    path is provided.
    """

    out = shutil.disk_usage(t_path)
    return out.free / 1024**3


def cli_question(text, valid_answers, default_answer=False, error_string=""):
    """
    Ask a multiple choice question at CLI and returb user choice

    Inputs
    text - string to present to user
    valid answers - list of valid answers that can be provided
    default_answer - an integer indicating which element in valid_answers is the default
    error_string - string to present if for some reason the user's response is not valid
    """

    reply = ""

    print("%s" % text)
    second_pass = False

    while True:

        if second_pass:
            if len(error_string) > 0:
                print(error_string)
            else:
                print(text)

        if default_answer is False:
            reply = input("? ")
        else:
            # If the user selects the default answer we just return this
            reply = input("[%s] ? " % (default_answer + 1))
            if len(reply) == 0:
                return valid_answers[default_answer]

        second_pass = True

        # If a valid reply is provided, we break and return it. Otherwise keep looping
        if reply in valid_answers:
            break

    return reply


def find_reg_dirs(verbose=True):
    """
    Looks for existing registration sub-directories

    Purpose
    Return absolute paths to existing registration sub-directories made by
    the function aratools.make_reg_dir

    Outputs
    existing_reg_dirs - list of absolute paths to all existing registration directories
    """

    reg_dir = os.path.abspath(ROOT_REG_DIR)
    existing_reg_dirs = list()

    if verbose:
        print("Looking for registration directories in path %s" % reg_dir)

    if not os.path.exists(reg_dir):
        if verbose:
            print("find_reg_dirs can not find path %s" % reg_dir)
        return existing_reg_dirs

    existing_sub_dirs = [x[0] for x in os.walk(reg_dir)]

    if len(existing_sub_dirs) == 0:
        if verbose:
            print("find_reg_dirs finds no sub-directories in path %s" % reg_dir)
        return existing_reg_dirs

    for t_dir in existing_sub_dirs:
        if re.match(REG_DIR_REGEX, os.path.basename(t_dir)):
            existing_reg_dirs.append(t_dir)

    return existing_reg_dirs


def make_reg_dir(simulate=False):
    """
    ARA helper function. Makes directories for registration within the current directory

    pathToRegDir = make_reg_dir

    Purpose
    Make a directory into which we will register data. Return the path as a string.
    Makes directories in the form:

    registration/reg_01__2020_03_19_a
    registration/reg_02__2020_03_19_b
    registration/reg_03__2020_03_20_a
    etc..

    Within each the following will be created by ARAregister:
    registration/reg_01__2020_03_19_a/ARA_to_sample
    registration/reg_01__2020_03_19_a/sample_to_ARA

    Outputs
    pathToRegDir - absolute path to the just-made registration directory

    """

    if simulate:
        print("Running tools.make_reg_dir()")

    reg_dir = os.path.abspath(ROOT_REG_DIR)

    if not os.path.exists(reg_dir):
        if simulate:
            print("Making directory %s" % reg_dir)
        else:
            os.mkdir(reg_dir)

    sub_dir_names = find_reg_dirs()

    # Make the registration sub-directory
    today_date = datetime.now().strftime("%Y_%m_%d")
    if not sub_dir_names:
        # If there are no registration sub-directories then we make one.
        reg_dir_to_make = "reg_01__%s_a" % today_date
    else:
        # Otherwise registrations already exist. We make a new directory that
        # increments the previous one:
        last_dir_name = os.path.basename(sub_dir_names[-1])

        reg_index = re.search(r"reg_(\d{2})", last_dir_name)

        # The number of directories that contain today's date
        n_dirs = len([d for d in sub_dir_names if today_date in d])

        reg_dir_to_make = "reg_%02d__%s_%s" % (
            int(reg_index.group(1)) + 1,
            today_date,
            increment_char("a", n_dirs),
        )

    full_path_to_make = os.path.join(reg_dir, reg_dir_to_make)

    if simulate:
        print("Making directory %s" % full_path_to_make)
    else:
        os.mkdir(full_path_to_make)

    return full_path_to_make


def increment_char(t_char, increment_by=1):
    """
    Increment character to next highest for making file names
    """

    o_char = ord(t_char)

    # Increment
    o_char += increment_by

    # 122 is "z" so next go to "A" if we are here
    if o_char > 122:
        o_char -= 58

    return chr(o_char)



"""
Unit tests for btpytools
Run with "pytest tests/tests/basic.py" from project root
"""


import unittest
import os
from os.path import join
from btpytools import transferToServer as tts
from btpytools import tools


DATA_DIR = join("tests", "data")
VALID_SAMPLE_DIR1 = join(DATA_DIR, "valid_sample_directory_01")
VALID_SAMPLE_DIR2 = join(DATA_DIR, "valid_sample_directory_02")
CROPPED_ACQ_DIR1 = join(DATA_DIR, "contains_data_subfolders_01")
CROPPED_ACQ_DIR2 = join(DATA_DIR, "contains_data_subfolders_02")


class TestTools(unittest.TestCase):
    """
    Run tests associated with btpytools.tools
    """

    def test_valid_directory_is_valid(self):
        """
        Confirm if folder defined by VALID_SAMPLE_DIR1 is indeed valid
        """
        self.assertTrue(tools.is_data_folder(VALID_SAMPLE_DIR1))

    def test_has_raw_data(self):
        """
        Confirm if VALID_SAMPLE_DIR1 contains a raw data folder
        """
        self.assertTrue(tools.has_raw_data(VALID_SAMPLE_DIR1))

    def test_has_compressed_raw_data(self):
        """
        Confirm if VALID_SAMPLE_DIR1 contains a compressed data folder
        """
        self.assertTrue(tools.has_compressed_raw_data(VALID_SAMPLE_DIR1))

    def test_has_recipe_file(self):
        """
        Confirm if VALID_SAMPLE_DIR1 contains a recipe file
        """
        self.assertTrue(tools.has_recipe_file(VALID_SAMPLE_DIR1))

    def test_has_scan_settings(self):
        """
        Confirm if VALID_SAMPLE_DIR1 has a scan settings file
        """
        self.assertTrue(tools.has_scan_settings(VALID_SAMPLE_DIR1))

    def test_has_stitched_images_directory(self):
        """
        Confirm if VALID_SAMPLE_DIR1 has a stitched images directory
        """
        self.assertTrue(tools.has_stitched_images_directory(VALID_SAMPLE_DIR1))

    def test_uncropped_stitched_images(self):
        """
        Confirm if VALID_SAMPLE_DIR1 has an uncropped data directory
        """
        self.assertTrue(tools.has_uncropped_stitched_images(CROPPED_ACQ_DIR1))

    def test_invalid_directories_are_invalid(self):
        """
        Confirm if that invalid directories are, indeed, invalid
        """
        invalid_dir = join(DATA_DIR, "invalid_data_dirs")

        for t_path in os.listdir(invalid_dir):
            self.assertFalse(tools.is_data_folder(join(invalid_dir, t_path)))

    def test_contains_data_subfolders(self):
        """
        Confirm that folder contains sub-folders that are data folders
        """
        self.assertTrue(tools.contains_data_folders(CROPPED_ACQ_DIR1))

    def test_contains_two_downsampled_dirs(self):
        """
        Confirm that the downsampled stacks directory in the valid sample has two downsampled
        stacks
        """
        out = tools.available_downsampled_volumes(VALID_SAMPLE_DIR1, verbose=True)
        self.assertTrue(len(out) == 4)


class TestTransferToServer(unittest.TestCase):
    """
    Run tests associated with btpytools.transferToServer.
    Some functions within this module are hard to understand without the tests
    """

    def test_check_directories_exist(self):
        """
        Confirm if that that two data directories exist
        """
        out = tts.check_directories(join(DATA_DIR, "dir_a"), join(DATA_DIR, "dir_b"))
        self.assertTrue(out)

    def test_dir_list_contains_compressed_archive(self):
        """
        Confirm that the list of paths contains at least one compressed archive
        """
        a_1 = ["dir1/dir2/stuff_rawData.tar.bz", "dir_a", "dir_b"]
        a_2 = ["dir1/dir2/stuff_rawData.tar.gz", "dir_a", "dir_b"]
        a_3 = [
            "dir1/dir2/stuff_rawData.tar.gz",
            "dir_a",
            "dir_b",
            "dir1/dir2/stuff_rawData.tar.bz",
        ]
        self.assertTrue(tts.dir_list_contains_compressed_archive(a_1))
        self.assertTrue(tts.dir_list_contains_compressed_archive(a_2))
        self.assertTrue(tts.dir_list_contains_compressed_archive(a_3))

    def test_dir_list_not_contains_compressed_archive(self):
        """
        Confirm that the list of paths contains no compressed archives
        """
        b_1 = ["dir1/dir2", "dir_a", "dir_b"]
        b_2 = ["dir1/dir2/stuff_rawData.tar", "dir_a", "dir_b"]
        b_3 = ["dir1/dir2/stuff_rawData", "dir_a", "dir_b", "dir1/dir2/rawData"]
        self.assertFalse(tts.dir_list_contains_compressed_archive(b_1))
        self.assertFalse(tts.dir_list_contains_compressed_archive(b_2))
        self.assertFalse(tts.dir_list_contains_compressed_archive(b_3))

    def test_user_specified_two_individual_dirs(self):
        """
        User specifies two cropped samples within a single acquisition
        """
        t_dirs = [join(CROPPED_ACQ_DIR1, "dir1"), join(CROPPED_ACQ_DIR1, "dir1")]
        self.assertTrue(tts.user_specified_cropped_directories_individually(t_dirs))

    def test_user_specifies_valid_directories(self):
        """
        Should return False because this is not a cropped sample
        """
        self.assertFalse(
            tts.user_specified_cropped_directories_individually(VALID_SAMPLE_DIR1)
        )

        self.assertFalse(
            tts.user_specified_cropped_directories_individually(
                [VALID_SAMPLE_DIR1, VALID_SAMPLE_DIR2]
            )
        )

    def test_user_mixes_individual_and_single_valid(self):
        """
        The user mixes indiviudally specified directories with "valid" samples directories.
        We should return true
        """
        t_dirs = [join(CROPPED_ACQ_DIR1, "dir1"), join(CROPPED_ACQ_DIR1, "dir1")]

        # With one valid sample directory at the list end
        t_1 = t_dirs
        t_1.append(VALID_SAMPLE_DIR1)
        self.assertTrue(tts.user_specified_cropped_directories_individually(t_1))

        # With one valid sample directory at the list start
        t_2 = t_dirs
        t_2.insert(0, VALID_SAMPLE_DIR1)
        self.assertTrue(tts.user_specified_cropped_directories_individually(t_2))

    def test_user_specifies_one_directory_from_a_cropped_acq(self):
        """
        Should return True because this is a cropped sample.
        """
        out = tts.user_specified_cropped_directories_individually(
            join(CROPPED_ACQ_DIR1, "dir1")
        )
        self.assertTrue(out)

    def test_identify_missed_raw_data_archive(self):
        """
        Check whether, we correctly identify a list of source directories as being problematic
        for not having raw data being sent.
        """
        f_1 = [join(CROPPED_ACQ_DIR1, "dir1"), join(CROPPED_ACQ_DIR1, "dir2")]
        self.assertTrue(tts.issue_warning_if_compressed_data_will_not_be_sent(f_1))

    def test_identify_acceptable_individually_specified_dirs(self):
        """
        Check whether directories specified individually are are nontheless OK
        """

        # Should pass because user supplies compressed archive
        t_1 = [
            join(CROPPED_ACQ_DIR1, "dir1"),
            join(CROPPED_ACQ_DIR1, "dir2"),
            join(CROPPED_ACQ_DIR1, "compressed_rawData.tar.bz"),
        ]

        self.assertFalse(tts.issue_warning_if_compressed_data_will_not_be_sent(t_1))

        # Should pass because specified dirs are in a directory with no compressed data
        crop_no_comp_dat = join(DATA_DIR, "contains_data_subfolders_no_compress_data")
        t_2 = [join(crop_no_comp_dat, "dir1"), join(crop_no_comp_dat, "dir2")]
        self.assertFalse(tts.issue_warning_if_compressed_data_will_not_be_sent(t_2))

    def test_compressed_data_warning_with_normal_call_structure(self):
        """
        Test whether a normal call structure (e.g. a cropped roor dir) will trigger no error
        """

        self.assertFalse(
            tts.issue_warning_if_compressed_data_will_not_be_sent(VALID_SAMPLE_DIR1)
        )
        self.assertFalse(
            tts.issue_warning_if_compressed_data_will_not_be_sent(CROPPED_ACQ_DIR1)
        )

        # Two directories containing cropped acquisitions should also pass
        t_1 = [CROPPED_ACQ_DIR1, CROPPED_ACQ_DIR2]
        self.assertFalse(tts.issue_warning_if_compressed_data_will_not_be_sent(t_1))

    def test_identify_peculiar_individually_specified_cases(self):
        """
        The user might mix and match individually specified directories with "normal" directories.
        Check these cases: when in doubt we just want to flag it.
        """
        f_1 = [
            join(CROPPED_ACQ_DIR1, "dir1"),
            join(CROPPED_ACQ_DIR1, "dir2"),
            VALID_SAMPLE_DIR1,
        ]

        f_2 = [
            VALID_SAMPLE_DIR1,
            join(CROPPED_ACQ_DIR1, "dir1"),
            join(CROPPED_ACQ_DIR1, "dir2"),
        ]
        self.assertTrue(tts.issue_warning_if_compressed_data_will_not_be_sent(f_1))
        self.assertTrue(
            tts.issue_warning_if_compressed_data_will_not_be_sent(f_2, verbose=True)
        )



"""
setup for btpytools
"""

import os
from setuptools import setup, find_packages

# Read in the README file as a variable so we can later add it to the description

THIS_DIRECTORY = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(THIS_DIRECTORY, "README.md"), encoding="utf-8") as f:
    LONG_DESCRIPTION = f.read()


# Blank setup.py which pip seems to need in order to be happy with setup.cfg
setup(
    name="btpytools",
    version="0.1.25",
    packages=find_packages(),
    description="Helper functions for BakingTray and StitchIt",
    license_files="LICENSE",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    install_requires=["pyyaml","brainreg"],
    extras_require={"dev": ["black", "pytest"]},
    python_requires=">=3.6",
    url="https://github.com/SainsburyWellcomeCentre/btpytools",
    project_urls={
        "Source Code": "https://github.com/SainsburyWellcomeCentre/btpytools"
    },
    author="Rob Campbell",
    author_email="rob.campbell@ucl.ac.uk",
    classifiers=[
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
    ],
    zip_safe=True,
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "summariseAcqs=btpytools.summariseAcquisitions:main",
            "compressRawData=btpytools.compressRawData:main",
            "transferToServer=btpytools.transferToServer:main",
            "brainregbt=btpytools.brainreg_bt:main",
        ]
    },
)



"""
Tools for handling BakingTray recipe files
"""

import os
import yaml

from glob import glob
from btpytools import tools


def read_recipe(dirToSearch=""):
    """Read recipe file from current directory or a defined directory.
    If multiple files present, reads the last one
    Return False if no recipe present
    """
    if len(dirToSearch) == 0:
        dirToSearch = os.getcwd()

    if tools.has_recipe_file(dirToSearch) == False:
        print("No recipe file found in", dirToSearch)
        return 0

    with open(glob(os.path.join(dirToSearch, "recipe*.yml"))[-1], "r") as stream:
        try:
            recipe = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print("Failed to read recipe file")
            print(exc)
            return False

    return recipe


def sample_id(dirToSearch=""):
    """return sample ID as a string"""
    this_recipe = read_recipe(dirToSearch)
    if not this_recipe:
        return False
    else:
        return this_recipe["sample"]["ID"]


def microscope(dirToSearch=""):
    """return microscope name as a string"""
    this_recipe = read_recipe(dirToSearch)
    if not this_recipe:
        return False
    else:
        return this_recipe["SYSTEM"]["ID"]


def acq_start_time(dirToSearch=""):
    """return acquisition start time as a string"""
    this_recipe = read_recipe(dirToSearch)
    if not this_recipe:
        return False
    else:
        return this_recipe["Acquisition"]["acqStartTime"]
